# Results for DSGE model
# this one is nice because it does all dep vars at once, without loop
# no CIs in this version, could add them
printf("\n\nDoing SBIL estimation using k-nearest neighbors nonparametric regression\n\n");
tic;
ann;
load simdata.out;
test = sum(isnan(simdata)+isinf(simdata),2);
test = (test ==0);
printf("\nSummary of data processing:\n");
printf("%d rows read in\n", rows(simdata));
simdata = simdata(test,:);
printf("%d rows after drop NaNs and Infs\n", rows(simdata));

insamp = simdata(:,2); # 0 for insamp (from param space), 1 for MC design
theta = simdata(:,3:10);
#Z = simdata(:,[11:columns(simdata)-9]) ;
Z = simdata(:,[11:25 26:28 29:34 35:43]) ; # choose which aux stats to use
Z = Z ./ repmat(std(Z),rows(Z),1);
# FIRST DESIGN true values
alpha   = 0.36;
beta    = 0.95;
delta   = 0.025;
eta 	= 0.2;
gam     = 2;
rho     = 0.85;
sigma   = 0.04;
nss 	= 1/3;
# SECOND DESIGN true values
# alpha   = 0.36;
# beta    = 0.95;
# delta   = 0.025;
# eta 	= 0.4;
# gam     = 3;
# rho     = 0.85;
# sigma   = 0.04;
# nss 	= 1/3;

# switch from nss to psi
kss = ((1/beta + delta - 1)/alpha/(nss^(1-alpha))) ^ (1/(alpha-1));
iss = delta*kss;
css = kss^alpha*nss^(1-alpha) - iss;
lss = (css-eta*css)^(-gam) - alpha*beta*(css-eta*css)^(-gam);
psi = (1 - alpha*beta)*((css-eta*css)^-gam)*(1-alpha)*(kss^alpha)*(nss^-alpha);
model_params0 = [alpha; beta; delta; eta; gam; rho; sigma; psi];

# find which are in sample (from param space)
test = (insamp == 0);
Z_in = Z(test,:);
theta_in = theta(test,:);
kd = ANNkd_tree(Z_in);
n = rows(Z_in);
q = columns(Z_in);
#k = floor(1.5*n^0.25);
k = floor(0.5*n^0.25);

# find the out of sample (at Monte Carlo design point)
test = (insamp == 1);
Z_out = Z(test,:);
theta_out = theta(test,:);
n = rows(Z_out);
contrib = zeros(n,8);
tolerance = 0;
for i = 1:n 
	[nn_idx,dd] = kd.annkPriSearch(Z_out(i,:), k, tolerance);
	nn_idx = nn_idx + 1; # ann using zero-based indexing
	keepers = theta_in(nn_idx,:);
	thetahat = mean(keepers);
	contrib(i,:) = thetahat;
endfor
m = mean(contrib);
s = std(contrib);
e = contrib - repmat(model_params0',rows(contrib),1);
b = mean(e);
e = e.^2;
mse = mean(e);
rmse = sqrt(mse);
priormean = mean(theta_in);
priorsdev = std(theta_in);
priorbias = priormean - model_params0';
priorrmse = sqrt(priorbias.^2 + priorsdev.^2);
mae = mean(abs(e));
rlabels = char("param", "mean", "pri_mean", "st.dev.","pri_sdev", "bias", "pri_bias","rmse", "pri_rmse");
clabels = char("alpha","beta","delta","eta", "gam", "rho","sigma", "psi");
printf("\n\nSBIL estimation results\n");
prettyprint([model_params0'; m; priormean; s; priorsdev; b ; priorbias; rmse; priorrmse], rlabels, clabels)
printf("number of Monte Carlo reps: %d\n", rows(Z_out));
printf("\n\n");

