# presently, fixed parameters are eta=0 and psi to make steady state hours
# equal to 1/3 time endowment
# adjust lower and upper bounds to free them

# this is to solve a model many times using different parameter values
# drawn from a prior distribution. This runs on a cluster. The frontend
# node (MPI rank 0) does not participate in the solution, it only collects
# the results and writes to disk. You should run this using something like
# mpirun -np X --hostfile /foo/bar octave -q --eval make_simdata
# where X is the total number of cores in the cluster, PLUS ONE. That way,
# all cores will be used in computations, and one will have the added duty
# of serving as the frontend

outfile = "simdata.out";
reps = 1e3;  # total replications (from param space + at design point) to generate
outsamp = 200; # desired (approximate) number of reps at design point
n_pooled = 1;  # number of runs accumulated before sent from nodes to frontend
verbose = 1;
if not(MPI_Initialized) MPI_Init; endif
CW = MPI_Comm_Load("NEWORLD");
node = MPI_Comm_rank(CW);
nodes = MPI_Comm_size(CW);
mytag = 48;

# Monte Carlo designs, uncomment one of them.
 alpha   = 0.36;
 beta    = 0.95;
 delta   = 0.025;
 eta 	= 0.2;
 gam     = 2;
 rho     = 0.85;
 sigma   = 0.04;
 nss 	= 1/3;

#alpha   = 0.36;
#beta    = 0.95;
#delta   = 0.025;
#eta 	= 0.4;
#gam     = 3;
#rho     = 0.85;
#sigma   = 0.04;
#nss 	= 1/3;

model_params0 = [alpha; beta; delta; eta; gam; rho; sigma; nss];

###############################################################################
############## you do not need to alter anything below this line ##############
###############################################################################
outsamp_proportion = outsamp/reps;

if node
	more_please = 1;
	while more_please
		# break it up to do intermediate writes
		for j = 1:n_pooled
			# re-establish the state of the RNG, dynare seems to synchronize this across MPI nodes
			if ((i>1) || (j>1))
				if     node==1
					load state1;
					rand("state", s);
				elseif node==2
					load state2;
					rand("state", s);
				elseif node==3
					load state3;
					rand("state", s);
				elseif node==4
					load state4;
					rand("state", s);
				elseif node==5
					load state5;
					rand("state", s);
				elseif node==6
					load state6;
					rand("state", s);
				elseif node==7
					load state7;
					rand("state", s);
				elseif node==8
					load state8;
					rand("state", s);
				elseif node==9
					load state9;
					rand("state", s);
				elseif node==10
					load state10;
					rand("state", s);
				elseif node==11
					load state11;
					rand("state", s);
				elseif node==12
					load state12;
					rand("state", s);
				elseif node==13
					load state13;
					rand("state", s);
				elseif node==14
					load state14;
					rand("state", s);
				elseif node==15
					load state15;
					rand("state", s);
				elseif node==16
					load state16;
					rand("state", s);
				elseif node==17
					load state17;
					rand("state", s);
				elseif node==18
					load state18;
					rand("state", s);
				elseif node==19
					load state19;
					rand("state", s);
				elseif node==20
					load state20;
					rand("state", s);
				elseif node==21
					load state21;
					rand("state", s);
				elseif node==22
					load state22;
					rand("state", s);
				elseif node==23
					load state23;
					rand("state", s);
				elseif node==24
					load state24;
					rand("state", s);
				elseif node==25
					load state25;
					rand("state", s);
				elseif node==26
					load state26;
					rand("state", s);
				elseif node==27
					load state27;
					rand("state", s);
				elseif node==28
					load state28;
					rand("state", s);
				elseif node==29
					load state29;
					rand("state", s);
				elseif node==30
					load state30;
					rand("state", s);
				elseif node==31
					load state31;
					rand("state", s);
				elseif node==32
					load state32;
					rand("state", s);
				endif
			endif
	
			# most draws will be from param space, but some from true values
			if (rand(1,1) < outsamp_proportion)
				model_params = model_params0;
				outsamp = 1;
			else
				lb = [0.25; 0.93; 0.02; 0; 1; 0.8; 0.01; 1/3];
				ub = [0.4; 0.99; 0.04; 0.5; 4; 0.99; 0.08; 1/3];
				model_params = rand(8,1).*(ub-lb) + lb;
				outsamp = 0;
			endif
			
			# break into pieces
			alpha = model_params(1,:);
			beta = model_params(2,:);
			delta = model_params(3,:);
			eta = model_params(4,:);
			gam = model_params(5,:);
			rho = model_params(6,:);
			sigma = model_params(7,:);
			nss = model_params(8,:);
			
			# there are a number of .mod files that each read a different parameter file
			# this writes out the different parameter files so that a number of solutions
			# can be done in parallel
			s = rand("state");
			if     node==1
				save parameterfile1  alpha beta delta eta gam rho sigma nss;
				save state1 s;
			elseif node==2
				save parameterfile2  alpha beta delta eta gam rho sigma nss;
				save state2 s;
			elseif node==3
				save parameterfile3  alpha beta delta eta gam rho sigma nss;
				save state3 s;
			elseif node==4
				save parameterfile4  alpha beta delta eta gam rho sigma nss;
				save state4 s;
			elseif node==5
				save parameterfile5  alpha beta delta eta gam rho sigma nss;
				save state5 s;
			elseif node==6
				save parameterfile6  alpha beta delta eta gam rho sigma nss;
				save state6 s;
			elseif node==7
				save parameterfile7  alpha beta delta eta gam rho sigma nss;
				save state7 s;
			elseif node==8
				save parameterfile8  alpha beta delta eta gam rho sigma nss;
				save state8 s;
			elseif node==9
				save parameterfile9  alpha beta delta eta gam rho sigma nss;
				save state9 s;
			elseif node==10
				save parameterfile10 alpha beta delta eta gam rho sigma nss;
				save state10 s;
			elseif node==11
				save parameterfile11 alpha beta delta eta gam rho sigma nss;
				save state11 s;
			elseif node==12
				save parameterfile12 alpha beta delta eta gam rho sigma nss;
				save state12 s;
			elseif node==13
				save parameterfile13 alpha beta delta eta gam rho sigma nss;
				save state13 s;
			elseif node==14
				save parameterfile14 alpha beta delta eta gam rho sigma nss;
				save state14 s;
			elseif node==15
				save parameterfile15 alpha beta delta eta gam rho sigma nss;
				save state15 s;
			elseif node==16
				save parameterfile16 alpha beta delta eta gam rho sigma nss;
				save state16 s;
			elseif node==17
				save parameterfile17 alpha beta delta eta gam rho sigma nss;
				save state17 s;
			elseif node==18
				save parameterfile18 alpha beta delta eta gam rho sigma nss;
				save state18 s;
			elseif node==19
				save parameterfile19 alpha beta delta eta gam rho sigma nss;
				save state19 s;
			elseif node==20
				save parameterfile20 alpha beta delta eta gam rho sigma nss;
				save state20 s;
			elseif node==21
				save parameterfile21 alpha beta delta eta gam rho sigma nss;
				save state21 s;
			elseif node==22
				save parameterfile22 alpha beta delta eta gam rho sigma nss;
				save state22 s;
			elseif node==23
				save parameterfile23 alpha beta delta eta gam rho sigma nss;
				save state23 s;
			elseif node==24
				save parameterfile24 alpha beta delta eta gam rho sigma nss;
				save state24 s;
			elseif node==25
				save parameterfile25 alpha beta delta eta gam rho sigma nss;
				save state25 s;
			elseif node==26
				save parameterfile26 alpha beta delta eta gam rho sigma nss;
				save state26 s;
			elseif node==27
				save parameterfile27 alpha beta delta eta gam rho sigma nss;
				save state27 s;
			elseif node==28
				save parameterfile28 alpha beta delta eta gam rho sigma nss;
				save state28 s;
			elseif node==29
				save parameterfile29 alpha beta delta eta gam rho sigma nss;
				save state29 s;
			elseif node==30
				save parameterfile30 alpha beta delta eta gam rho sigma nss;
				save state30 s;
			elseif node==31
				save parameterfile31 alpha beta delta eta gam rho sigma nss;
				save state31 s;
			elseif node==32
				save parameterfile32 alpha beta delta eta gam rho sigma nss;
				save state32 s;
			endif
	
			# solve model, using the appropriate parameter file
			if node==1
				dynare RM_HABIT1 noclearall;
			elseif node==2
				dynare RM_HABIT2 noclearall; 
			elseif node==3
				dynare RM_HABIT3 noclearall;
			elseif node==4
				dynare RM_HABIT4 noclearall;
			elseif node==5
				dynare RM_HABIT5 noclearall;
			elseif node==6
				dynare RM_HABIT6 noclearall;
			elseif node==7
				dynare RM_HABIT7 noclearall;
			elseif node==8
				dynare RM_HABIT8 noclearall;
			elseif node==9
				dynare RM_HABIT9 noclearall;
			elseif node==10
				dynare RM_HABIT10 noclearall;
			elseif node==11
				dynare RM_HABIT11 noclearall;
			elseif node==12
				dynare RM_HABIT12 noclearall;
			elseif node==13
				dynare RM_HABIT13 noclearall;
			elseif node==14
				dynare RM_HABIT14 noclearall;
			elseif node==15
				dynare RM_HABIT15 noclearall;
			elseif node==16
				dynare RM_HABIT16 noclearall;
			elseif node==17
				dynare RM_HABIT17 noclearall;
			elseif node==18
				dynare RM_HABIT18 noclearall;
			elseif node==19
				dynare RM_HABIT19 noclearall;
			elseif node==20
				dynare RM_HABIT20 noclearall;
			elseif node==21
				dynare RM_HABIT21 noclearall;
			elseif node==22
				dynare RM_HABIT22 noclearall;
			elseif node==23
				dynare RM_HABIT23 noclearall;
			elseif node==24
				dynare RM_HABIT24 noclearall;
			elseif node==25
				dynare RM_HABIT25 noclearall;
			elseif node==26
				dynare RM_HABIT26 noclearall;
			elseif node==27
				dynare RM_HABIT27 noclearall;
			elseif node==28
				dynare RM_HABIT28 noclearall;
			elseif node==29
				dynare RM_HABIT29 noclearall;
			elseif node==30
				dynare RM_HABIT30 noclearall;
			elseif node==31
				dynare RM_HABIT31 noclearall;
			elseif node==32
				dynare RM_HABIT32 noclearall;
			endif
			# get a simulation of length 80 (20 years quarterly), and compute aux. statistic
			data = [c i n] ;
			data = data(101:260,:);
			#Z = [aux_stat(data); oo_.mean;  sqrt(diag(oo_.var));  vec(oo_.autocorr{1}); vec(oo_.autocorr{2})];
			Z = [aux_stat(data); oo_.mean; vech(oo_.var);  vec(oo_.autocorr{1}); vec(oo_.autocorr{2}); vec(oo_.autocorr{3})];
			
			# switch from nss to psi
			kss = ((1/beta + delta - 1)/alpha/(nss^(1-alpha))) ^ (1/(alpha-1));
			iss = delta*kss;
			css = kss^alpha*nss^(1-alpha) - iss;
			lss = (css-eta*css)^(-gam) - alpha*beta*(css-eta*css)^(-gam);
			psi = (1 - alpha*beta)*((css-eta*css)^-gam)*(1-alpha)*(kss^alpha)*(nss^-alpha);
			model_params(8,:) = psi;
			
			contrib = [node outsamp model_params' Z'];
			if (i==1) & (j==1) contribs = zeros(n_pooled, columns(contrib)); endif
			contribs(j,:) = contrib;	
		endfor
		MPI_Send(contribs, 0, mytag, CW);
		pause(0.1); # give time for the fronted to send a stop message, if done
		# check if we're done
		if (MPI_Iprobe(0, node, CW)) # check for ping from rank 0
			junk = MPI_Recv(0, node, CW);
			break;
		endif
	endwhile
else # frontend
	received = 0;
	done = false;
	while received < reps
		# retrieve results from compute nodes
		pause(0.1);
		for i = 1:nodes-1
			# compute nodes have results yet?
			ready = false;
			ready = MPI_Iprobe(i, mytag, CW); # check if message pending
			if ready
				# get it if it's there
				contribs = MPI_Recv(i, mytag, CW);
				need = reps - received;
				received = received + n_pooled;
				# truncate?
				if n_pooled  >= need
						contribs = contribs(1:need,:);
						done = true;
				endif
				# write to output file
				FN = fopen (outfile, "a");
				if (FN < 0) error ("make_simdata: couldn't open output file %s", outfile); endif
				for j = 1:rows(contribs)
					fprintf(FN, "%f ", contribs(j,:));
					fprintf(FN, "\n");
				endfor
				fclose(FN);
				if verbose 
					printf("\nContribution received from node%d.  Received so far: %d\n", i, received);
				endif
				if done
					# tell compute nodes to stop loop
					for j = 1:5
						for i = 1:(nodes-1)
							if (j==1) MPI_Send(" ",i,i,CW); endif # send out message to stop
							ready = MPI_Iprobe(i, mytag, CW); # get last messages
							if ready contribs = MPI_Recv(i, mytag, CW); endif
						endfor
					endfor
					break;
				endif	
			endif
		endfor
	endwhile
endif
if not(MPI_Finalized) MPI_Finalize; endif
